/**
 * creator.js
 * creates document elements for html page
 */

/**
 * creates a table of results from endpoint
 * a row is one result
 * each column is value of one result
 * table head contains selected values
 */
function createResultsTable() {

    if (typeof queryResults === "undefined" || queryResults === "") {
        return;
    }

    var but = document.getElementById("table-results-button");
    but.className = but.className + " active";

    document.getElementById("json-results-button").className = "btn btn-secondary";


    var d = document.getElementById("results-table");
    document.getElementById("results-json").innerHTML = "";

    d.innerHTML = "";

    var resultsTable = document.createElement("table");
    resultsTable.className = "table table-hover table-striped table-bordered";

    var vars = queryResults.head.vars;

    var csv = "";


    var thead = resultsTable.insertRow(-1);

    for (var i = 0; i < vars.length; i++) {
        createTableHead(thead, vars[i]);
        csv += vars[i] + ",";
    }

    csv += "\n";

    var bindings = queryResults.results.bindings;

    for (var j = 0; j < bindings.length; j++) {

        var row = resultsTable.insertRow(-1);

        for (var s = 0; s < vars.length; s++) {

            var cell = row.insertCell(-1);

            var val = bindings[j][vars[s]];

            // can be OPTIONAL and not present !
            if (typeof val === "undefined") {
                cell.innerHTML = "";

                csv += ",";

            } else {

                if (val.type === "uri") {

                    var annotation = getAnnotations(val.value);

                    if (annotation !== "") {

                        cell.innerHTML = annotation;

                        csv += annotation + ",";

                    } else {
                        cell.innerHTML = val.value;

                        csv += val.value + ",";
                    }


                } else {
                    cell.innerHTML = val.value;

                    csv += val.value + ",";

                }

            }


        }

        csv += "\n";

    }


    d.appendChild(resultsTable);

    return csv;

}

/**
 * returns text representation of uri or empty string
 * @param uri
 * @returns {*|boolean|string}
 */
function getAnnotations(uri) {

    // unless changed - form:code, rdfs:label, URI
    var predefinedLabels = document.getElementById("input-attributes").value;
    // cs,en,C
    var predefinedLanguages = document.getElementById("input-languages").value;

    var labels = predefinedLabels.split(",");
    var languages = predefinedLanguages.split(",");

    var annotation = "";

    if (!uri) {
        return annotation;
    }

    var nameSpace;
    if (uri.indexOf("#") > -1) {
        nameSpace = uri.substring(0, uri.indexOf("#"));
    } else {
        nameSpace = uri;
    }

    var xml = getOntologyByNameSpace(nameSpace);

    if (!xml) {
        return annotation;
    }

    var element = getElementByAttributeValue(xml, "rdf:about", uri);

    if (!element) {
        return annotation;
    }

    var loaded = [];

    for (var i = 0; i < labels.length; i++) {

        for (var o = 0; o < element.childNodes.length; o++) {

            var child = element.childNodes[o];

            if (child.nodeName === labels[i]) {

                if (child.attributes.length > 0) {

                    loaded.push({
                        label: labels[i],
                        value: child.textContent,
                        language: child.attributes[0].value
                    });
                } else {
                    loaded.push({
                        label: labels[i],
                        value: child.textContent,
                        language: "C"
                    });
                }
            }


        }
    }


    for (var l = 0; l < labels.length; l++) {

        for (var p = 0; p < languages.length; p++) {

            var loadedPriority = getLoadedByLabelLanguage(loaded, labels[l], languages[p]);

            if (loadedPriority !== false) {
                return loadedPriority;
            }

        }
    }


    return annotation;


}

function getLoadedByLabelLanguage(loaded, label, language) {
    for (var s = 0; s < loaded.length; s++) {
        if (loaded[s].label === label && loaded[s].language === language) {
            return loaded[s].value;
        }
    }

    return false;
}


/**
 * creates a JSON results
 */
function createResultsJSON() {

    if (typeof queryResults === "undefined" || queryResults === "") {
        return;
    }

    var but = document.getElementById("json-results-button");
    but.className = but.className + " active";

    document.getElementById("table-results-button").className = "btn btn-secondary";

    var d = document.getElementById("results-json");
    document.getElementById("results-table").innerHTML = "";

    d.innerHTML = "";

    var pre = document.createElement("pre");

    pre.innerHTML = JSON.stringify(queryResults, null, "\t");

    d.appendChild(pre);


}

function createObjectPropertiesTableHead(table) {
    var thead = table.insertRow(-1);

    createTableHead(thead, "Object property");
    createTableHead(thead, "Remove");

}

function createObjectPropertiesTable() {
    var d = document.getElementById("object-properties-table-div");

    d.innerHTML = "";

    var objectPropertiesTable = document.createElement("table");
    objectPropertiesTable.className = "table table-hover";

    createObjectPropertiesTableHead(objectPropertiesTable);

    var cls = classes[currentClass];

    for (var i = 0; i < cls.objectProperties.length; i++) {

        var property = cls.objectProperties[i];

        if (property.selected === false) {
            continue;
        }

        var tBody = objectPropertiesTable.insertRow(-1);

        var cell1 = tBody.insertCell(0);

        var target = classes[property.target];

        var text = property.prefixedLocalName + " => " + target.prefix + ":" + target.localName + " ";

        for (var s = 0; s < property.moreInfo.length; s++) {
            if (property.moreInfo[s].name === "label" || property.moreInfo[s].name === "title" || property.moreInfo[s].name === "prefLabel") {
                text += "[" + property.moreInfo[s].value + "] ";
            }
        }

        cell1.innerHTML = text;


        var cell2 = tBody.insertCell(1);

        var remove = document.createElement("button");
        remove.setAttribute("type", "button");
        remove.className = "btn btn-default btn-sm";
        remove.setAttribute("onclick", "cancelObjectProperty(" + i + ")");

        var glyph = document.createElement("span");
        glyph.className = "glyphicon glyphicon-remove";

        remove.appendChild(glyph);
        cell2.appendChild(remove);

    }

    d.appendChild(objectPropertiesTable);

}

/**
 *
 * @param table
 */
function createAggregationTableHead(table) {
    var thead = table.insertRow(-1);

    createTableHead(thead, "Aggregate function");
    createTableHead(thead, "Remove");
}

/**
 * creates aggregations table
 * selected aggregates are present in the table
 * each row has aggregation function and remove button
 * @param attribute
 */
function createAggregationsTable(attribute) {

    var d = document.getElementById("aggregations-table");

    d.innerHTML = "";

    var aggregationTable = document.createElement("table");
    aggregationTable.className = "table table-hover";

    createAggregationTableHead(aggregationTable);

    for (var i = 0; i < attribute.aggregations.length; i++) {

        var aggregation = attribute.aggregations[i];

        if (aggregation.selected === false) {
            continue;
        }

        var tBody = aggregationTable.insertRow(-1);

        var cell1 = tBody.insertCell(0);

        cell1.innerHTML = aggregation.aggr;


        var cell2 = tBody.insertCell(1);

        var remove = document.createElement("button");
        remove.setAttribute("type", "button");
        remove.className = "btn btn-default btn-sm";
        remove.setAttribute("onclick", "cancelAggregation(" + i + ")");

        var glyph = document.createElement("span");
        glyph.className = "glyphicon glyphicon-remove";

        remove.appendChild(glyph);
        cell2.appendChild(remove);

    }

    d.appendChild(aggregationTable);


}

/**
 * creates an object for aggregation functions
 * @param attribute
 */
function createAggregationsOptions(attribute) {

    for (var i = 0; i < aggregationsOptions.length; i++) {

        var aggr = {
            selected: false,
            attribute: attribute.localName,
            aggr: aggregationsOptions[i],
            variable: aggregationsOptions[i].toLowerCase() + "_" + attribute.localName
        };

        attribute.aggregations.push(aggr);
    }
}

/**
 * creates a selection for aggregations
 * only not selected ones are present
 * @param attribute
 */
function createAggregationsSelection(attribute) {

    var d = document.getElementById("aggregations-selection");

    d.innerHTML = "";

    if (attribute.aggregations.length === 0) {
        createAggregationsOptions(attribute);
    }

    var options = [];


    var blank = document.createElement("option");
    document.getElementById("aggregations-selection").appendChild(blank);

    for (var i = 0; i < attribute.aggregations.length; i++) {

        var aggr = attribute.aggregations[i];

        if (aggr.selected === true) {
            continue;
        }

        var option = {
            text: aggr.aggr,
            id: i,
        };

        options.push(option);
    }

    $(".aggregations-selection").select2({
        placeholder: "Add aggregate function",
        data: options,
        sorter: function (data) {
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });


}

/**
 * creates a selection for filters
 * the result is actually a button to add a new filter
 * @param attribute
 */
function createFilterSelection(attribute) {

    var d = document.getElementById("filter-button");

    d.innerHTML = "";

    var btn = document.createElement("input");
    btn.setAttribute("type", "button");
    btn.className = "btn btn-info";
    btn.value = "Add a filter";

    btn.setAttribute("onclick", "addFilter(" + attribute.id + ")");

    d.appendChild(btn);
}

/**
 * clears aggregation and filters selection and table
 */
function clearFiltersAndAggregations() {
    document.getElementById("filter-button").innerHTML = "";
    document.getElementById("filter-table").innerHTML = "";
    document.getElementById("aggregations-selection").innerHTML = "";
    document.getElementById("aggregations-table").innerHTML = "";
}

/**
 *
 * @param table
 */
function createOrderByTableHead(table) {
    var thead = table.insertRow(-1);

    createTableHead(thead, "Variable");
    createTableHead(thead, "Ordering");
    createTableHead(thead, "Sort");
    createTableHead(thead, "Remove");
}

/**
 * creates a table for ordered values
 * selected values for ordering are shown
 * each row contains the ordered value, sorting modification (asc/desc), arrows to modify the variable ordering and button to remove ordering
 */
function createOrderByTable() {

    var d = document.getElementById("order-by-table");
    d.innerHTML = "";

    var orderByTable = document.createElement("table");
    orderByTable.className = "table table-hover order-table";

    createOrderByTableHead(orderByTable);


    for (var i = 0; i < orderBy.length; i++) {

        var order = orderBy[i];

        var trBody = orderByTable.insertRow(-1);

        var cell1 = trBody.insertCell(0);

        cell1.innerHTML = "?" + order.variable;

        var cell2 = trBody.insertCell(1);

        var selectOperators = document.createElement("select");
        selectOperators.className = "form-control";
        selectOperators.setAttribute("id", "" + i);

        for (var j = 0; j < order.orderOptions.length; j++) {
            var option = document.createElement("option");
            option.innerHTML = order.orderOptions[j];
            option.value = j;

            if (order.selectedOrderOption === j) {
                option.selected = true;
            }

            selectOperators.appendChild(option);

        }

        selectOperators.setAttribute("onchange", "selectOrderOption(this)");

        cell2.appendChild(selectOperators);

        var trCell3 = trBody.insertCell(2);

        var up = document.createElement("button");
        up.setAttribute("type", "button");
        up.className = "btn btn-default btn-sm";
        up.setAttribute("onclick", "orderUp(" + i + ")");

        var glyphUp = document.createElement("span");
        glyphUp.className = "glyphicon glyphicon-arrow-up";

        up.appendChild(glyphUp);

        var down = document.createElement("button");
        down.setAttribute("type", "button");
        down.className = "btn btn-default btn-sm";
        down.setAttribute("onclick", "orderDown(" + i + ")");

        var glyphDown = document.createElement("span");
        glyphDown.className = "glyphicon glyphicon-arrow-down";

        down.appendChild(glyphDown);

        trCell3.appendChild(up);
        trCell3.appendChild(down);

        var trCell4 = trBody.insertCell(3);

        var remove = document.createElement("button");
        remove.setAttribute("type", "button");
        remove.className = "btn btn-default btn-sm";
        remove.setAttribute("onclick", "cancelOrder(" + i + ")");

        var glyph = document.createElement("span");
        glyph.className = "glyphicon glyphicon-remove";

        remove.appendChild(glyph);
        trCell4.appendChild(remove);
    }

    d.appendChild(orderByTable);

}

/**
 * swaps ordered values with each other based on their index
 * @param oldIndex
 * @param newIndex
 */
function swap(oldIndex, newIndex) {
    var old = orderBy[oldIndex];

    orderBy[oldIndex] = orderBy[newIndex];
    orderBy[newIndex] = old;

}

/**
 * creates a filter table
 * each row contains filter operator, value and button to remove
 * @param attribute
 */
function createFilterTable(attribute) {
    var d = document.getElementById("filter-table");

    d.innerHTML = "";

    var filterTable = document.createElement("table");
    filterTable.className = "table table-hover";

    createFilterTableHead(filterTable);

    for (var i = 0; i < attribute.filters.length; i++) {

        var filter = attribute.filters[i];

        var tBody = filterTable.insertRow(-1);

        var cell1 = tBody.insertCell(0);

        var selectOperators = document.createElement("select");
        selectOperators.className = "form-control";
        selectOperators.setAttribute("id", "" + i);
        for (var j = 0; j < filter.options.length; j++) {
            var option = document.createElement("option");
            option.innerHTML = filter.options[j];
            option.value = j;

            if (filter.selectedOption === j) {
                option.selected = true;
            }

            selectOperators.appendChild(option);

        }

        selectOperators.setAttribute("onchange", "selectOperator(this)");

        cell1.appendChild(selectOperators);

        var cell2 = tBody.insertCell(1);

        var valueInput = document.createElement("input");
        valueInput.setAttribute("type", "text");
        valueInput.className = "form-control";

        valueInput.value = filter.value;

        valueInput.setAttribute("id", "" + i);
        valueInput.setAttribute("onkeyup", "filterValue(this)");

        cell2.appendChild(valueInput);

        var trCell3 = tBody.insertCell(2);

        var remove = document.createElement("button");
        remove.setAttribute("type", "button");
        remove.className = "btn btn-default btn-sm";
        remove.setAttribute("onclick", "cancelFilter(" + i + ")");

        var glyph = document.createElement("span");
        glyph.className = "glyphicon glyphicon-remove";

        remove.appendChild(glyph);
        trCell3.appendChild(remove);

    }

    d.appendChild(filterTable);

}

/**
 *
 * @param table
 */
function createFilterTableHead(table) {
    var thead = table.insertRow(-1);

    createTableHead(thead, "Operator");
    createTableHead(thead, "Value");
    createTableHead(thead, "Remove");
}

/**
 *
 * @param tr
 * @param desc
 */
function createTableHead(tr, desc) {

    var th = tr.insertCell(-1);
    th.innerHTML = "<strong>" + desc + "</strong>";
}

/**
 *
 * @param table
 */
function createAttributeTableHead(table) {

    var thead = table.insertRow((-1));

    createTableHead(thead, "Name");
    createTableHead(thead, "Count");
    createTableHead(thead, "Show");
    createTableHead(thead, "Optional");
    createTableHead(thead, "Select");
    createTableHead(thead, "Remove");
}

/**
 * creates an attribute table
 * only selected attributes are present
 * each row contains attribute name, attribute count, button to toggle SHOW, button to toggle OPTIONAL, button to select attribute and button to remove
 */
function createAttributeTable() {

    var d = document.getElementById("attribute-table-div");

    d.innerHTML = "";

    var classInfo = classes[currentClass];

    var attributeTable = document.createElement("table");
    attributeTable.className = "table table-hover";
    attributeTable.id = "attribute-table";

    createAttributeTableHead(attributeTable);

    for (var i = 0; i < classInfo.attributes.length; i++) {

        var attribute = classInfo.attributes[i];

        if (typeof attribute.id === "undefined") {
            attribute.id = i;
        }

        if (attribute.selected === true) {

            // add table row
            var trBody = attributeTable.insertRow(-1);

            if (attribute.clicked === true) {
                trBody.className = trBody.className + " active";
            } else if (getColour(attribute, classes[currentClass]) === "red") {
                trBody.className = trBody.className + " danger";
            } else if (getColour(attribute, classes[currentClass]) === "green") {
                trBody.className = trBody.className + " success";
            } else if (getColour(attribute, classes[currentClass]) === "yellow") {
                trBody.className = trBody.className + " warning";
            }


            var trCell1 = trBody.insertCell(0);
            trCell1.innerHTML = attribute.prefixedLocalName;

            var trCell12 = trBody.insertCell(1);
            trCell12.innerHTML = attribute.count;

            var trCell3 = trBody.insertCell(2);
            var showCheckBox = document.createElement("input");
            showCheckBox.setAttribute("type", "checkbox");
            showCheckBox.setAttribute("value", "" + i);
            if (attribute.show === true) {
                showCheckBox.checked = true;
            }

            showCheckBox.setAttribute("onClick", "showCheckBox(this)");
            trCell3.appendChild(showCheckBox);

            var trCell4 = trBody.insertCell(3);
            var optionalCheckBox = document.createElement("input");
            optionalCheckBox.setAttribute("type", "checkbox");
            optionalCheckBox.setAttribute("value", "" + i);
            if (attribute.optional === true) {
                optionalCheckBox.checked = true;
            }

            optionalCheckBox.setAttribute("onClick", "optionalCheckBox(this)");
            trCell4.appendChild(optionalCheckBox);


            var trCell6 = trBody.insertCell(4);

            var select = document.createElement("button");
            select.setAttribute("type", "button");
            select.className = "btn btn-default btn-sm";
            select.setAttribute("onclick", "addAttribute(" + attribute.id + ")");

            var glyph2 = document.createElement("span");
            glyph2.className = "glyphicon glyphicon-ok";

            select.appendChild(glyph2);
            trCell6.appendChild(select);


            var trCell5 = trBody.insertCell(5);

            var remove = document.createElement("button");
            remove.setAttribute("type", "button");
            remove.className = "btn btn-default btn-sm";
            remove.setAttribute("onclick", "cancelAttribute(" + attribute.id + ")");


            var glyph = document.createElement("span");
            glyph.className = "glyphicon glyphicon-remove";

            remove.appendChild(glyph);
            trCell5.appendChild(remove);

        }
    }

    d.appendChild(attributeTable);

}

/**
 * creates object properties selection
 * only not selected object properties are shown
 */
function createObjectPropertiesSelection() {

    var options = [];

    document.getElementById("object-properties-selection").options.length = 0;


    var blank = document.createElement("option");
    document.getElementById("object-properties-selection").appendChild(blank);


    for (var i = 0; i < classes[currentClass].objectProperties.length; i++) {
        var classInfo = classes[currentClass].objectProperties[i];

        var target = classes[classInfo.target];

        if (classInfo.selected === true) {
            continue;
        }

        var text = classInfo.prefixedLocalName + " => " + target.prefix + ":" + target.localName + " ";

        for (var s = 0; s < classInfo.moreInfo.length; s++) {
            if (classInfo.moreInfo[s].name === "label" || classInfo.moreInfo[s].name === "title" || classInfo.moreInfo[s].name === "prefLabel") {
                text += "[" + classInfo.moreInfo[s].value + "] ";
            }
        }

        var option = {
            text: text += " (" + classInfo.count + ")",
            id: i
        };

        options.push(option);
    }

    $(".object-properties-selection").select2({
        placeholder: "Select an object property",
        data: options,
        sorter: function (data) {
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });
}

/**
 * creates selection for ordering variables
 * all used variables in the query not already selected for ordering are present
 * @param variables
 */
function createOrderBySelection(variables) {

    var options = [];

    document.getElementById("order-by-selection").options.length = 0;


    var blank = document.createElement("option");
    document.getElementById("order-by-selection").appendChild(blank);

    for (var i = 0; i < variables.length; i++) {

        if (isVariableOrdered(variables[i]) !== false) {
            continue;
        }

        var option = {
            text: "?" + variables[i],
            id: variables[i],
        };

        options.push(option);
    }

    $(".order-by-selection").select2({
        placeholder: "Set ordering for variables",
        data: options,
        sorter: function (data) {
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });

}


/**
 * creates selection for attributes
 * only those not selected are present
 */
function createAttributesSelection() {

    var options = [];

    document.getElementById("attributes-selection").options.length = 0;

    var blank = document.createElement("option");
    document.getElementById("attributes-selection").appendChild(blank);

    for (var i = 0; i < classes[currentClass].attributes.length; i++) {
        var classInfo = classes[currentClass].attributes[i];

        if (classInfo.selected === true) {
            continue;
        }

        var text = classInfo.prefixedLocalName + " ";

        for (var s = 0; s < classInfo.moreInfo.length; s++) {
            if (classInfo.moreInfo[s].name === "label" || classInfo.moreInfo[s].name === "title" || classInfo.moreInfo[s].name === "prefLabel") {
                text += "[" + classInfo.moreInfo[s].value + "] ";
            }
        }

        if (classInfo.annotation === true) {
            text += " (annoptation)";
        }

        text += " (" + classInfo.count + ")";


        var option = {
            text: text,
            id: i
        };

        options.push(option);
    }

    $(".attributes-selection").select2({
        placeholder: "Select an attribute",
        data: options,
        sorter: function (data) {
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });

}

/**
 * creates selection of all attributes
 * only those not selected are present
 */
function createAllAttributesSelection() {

    var options = [];

    $('#select-attributes-selection').empty();

    var blank = document.createElement("option");
    document.getElementById("select-attributes-selection").appendChild(blank);

    for (var i = 0; i < classes.length; i++) {

        var classInfo = classes[i];

        var attributes = classInfo.attributes;

        var optgroupText = classInfo.prefixedLocalName;

        var optgroup = {
            text: optgroupText,
            children: []
        };

        for (var j = 0; j < attributes.length; j++) {

            var attribute = attributes[j];

            if (attribute.selected === true) {
                continue;
            }

            var text = attribute.prefixedLocalName + " ";

            for (var s = 0; s < attribute.moreInfo.length; s++) {
                if (attribute.moreInfo[s].name === "label" || attribute.moreInfo[s].name === "title" || attribute.moreInfo[s].name === "prefLabel") {
                    text += "[" + attribute.moreInfo[s].value + "] ";
                }
            }

            if (attribute.annotation === true) {
                text += " (annoptation)";
            }

            text += " (" + attribute.count + ")";

            var option = {
                id: classInfo.prefixedLocalName + "/" + j,
                text: text
            };

            optgroup.children.push(option);

        }

        options.push(optgroup);

    }

    $(".select-attributes-selection").select2({
        placeholder: "Select an attribute",
        data: options,
        sorter: function (data) {
            /* Sort data using lowercase comparison */
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });

}

/**
 * returns colour which to use in attribute table to differentiate between attributes
 * white - annotation or rdf:type
 * yellow - no information on domain
 * green - attribute is in the right class (class uri is in attribute domain)
 * red - attribute domain does not contain class uri
 * @param attribute
 * @param cls
 * @returns {string}
 */
function getColour(attribute, cls) {

    if (typeof attribute.domain == "undefined") {

        // we dont know any domain?
        //warning

        //is it
        if (attribute.localName === "title" || attribute.localName === "label" || attribute.localName === "prefLabel" || attribute.localName === "type") {
            return "white";
        } else {
            return "yellow";
        }


    } else if (attribute.domain.length > 0) {
        for (var z = 0; z < attribute.domain.length; z++) {

            if (attribute.domain[z] === cls.uri) {
                // right domain
                return "green";
            }
            if (attribute.domain[z] === cls.subClassOf) {
                return "green";
            }
        }

        // after for and we are here -> bad!
        return "red";
    } else {
        return "white";
    }

}

/**
 * creates a selection of classes
 * even those selected are present
 * choosing a selected class creates its duplicate
 */
function createClassSelection() {

    var options = [];

    document.getElementById("select-classes-selection").options.length = 0;

    var blank = document.createElement("option");
    document.getElementById("select-classes-selection").appendChild(blank);

    for (var i = 0; i < classes.length; i++) {


        var classInfo = classes[i];

        if (classInfo.duplicate === true) {
            continue;
        }

        var text = classInfo.prefixedLocalName + " ";

        for (var s = 0; s < classInfo.moreInfo.length; s++) {
            if (classInfo.moreInfo[s].name === "label" || classInfo.moreInfo[s].name === "title" || classInfo.moreInfo[s].name === "prefLabel") {
                text += "[" + classInfo.moreInfo[s].value + "] ";
            }
        }

        text += " (" + classInfo.count + ")";


        var option = {
            id: i,
            text: text
        };

        options.push(option);
    }

    $(".select-classes-selection").select2({
        placeholder: "Select a class",
        data: options,
        sorter: function (data) {
            return data.sort(function (a, b) {
                a = a.text.toLowerCase();
                b = b.text.toLowerCase();
                if (a > b) {
                    return 1;
                } else if (a < b) {
                    return -1;
                }
                return 0;
            });
        }
    });

}

/**
 * creates information panel
 * it is in fact a table, it always contains class name, prefixed name, URI, namespace and count
 * it can cotain additional information in the form of annotation properties
 */
function showInfo() {

    var d = document.getElementById("info");

    d.innerHTML = "";

    if (currentClass === -1) {
        return;
    }

    var classInfo = classes[currentClass];

    var infoTable = document.createElement("table");
    infoTable.className = "table table-condensed";

    var index = 0;
    showInfoAddRow(infoTable, index++, "Name", classInfo.localName);
    showInfoAddRow(infoTable, index++, "Prefixed name", classInfo.prefixedLocalName);
    showInfoAddRow(infoTable, index++, "URI", classInfo.uri);
    showInfoAddRow(infoTable, index++, "Name Space", classInfo.nameSpace);

    for (var i = 0; i < classInfo.moreInfo.length; i++) {

        showInfoAddRow(infoTable, index++, classInfo.moreInfo[i].name + " " + classInfo.moreInfo[i].tag.toUpperCase(), classInfo.moreInfo[i].value);
    }
    showInfoAddRow(infoTable, index++, "Count", classInfo.count);

    var showRow = infoTable.insertRow(-1);

    var cellDesc = showRow.insertCell(-1);
    cellDesc.innerHTML = "<strong>Show</strong>";

    var cellShow = showRow.insertCell(-1);

    var showCheckBox = document.createElement("input");
    showCheckBox.setAttribute("type", "checkbox");
    showCheckBox.setAttribute("value", "" + currentClass);
    if (classes[currentClass].show === true) {
        showCheckBox.checked = true;
    }

    showCheckBox.setAttribute("onClick", "showCheckboxClass(this)");

    cellShow.appendChild(showCheckBox);

    var countRow = infoTable.insertRow(-1);

    var cellDsc = countRow.insertCell(-1);
    cellDsc.innerHTML = "<strong>COUNT</strong>";

    var cellCount = countRow.insertCell(-1);

    var countCheckbox = document.createElement("input");
    countCheckbox.setAttribute("type", "checkbox");
    countCheckbox.setAttribute("value", "" + currentClass);
    if (classes[currentClass].aggrCount === true) {
        countCheckbox.checked = true;
    }

    countCheckbox.setAttribute("onClick", "countCheckbox(this)");

    cellCount.appendChild(countCheckbox);

    var duplicateRow = infoTable.insertRow(-1);
    var duplicateCell = duplicateRow.insertCell(-1);

    var duplicateButton = document.createElement("button");
    duplicateButton.className = "btn btn-info";

    duplicateButton.innerHTML = "Duplicate";
    duplicateButton.setAttribute("onclick", "duplicateClass(false)");

    duplicateCell.appendChild(duplicateButton);

    var duplicateRelationsCell = duplicateRow.insertCell(-1);
    var duplicateRelationsButton = document.createElement("button");
    duplicateRelationsButton.className = "btn btn-info";

    duplicateRelationsButton.innerHTML = "Duplicate with relations";
    duplicateRelationsButton.setAttribute("onclick", "duplicateClass(true)");

    duplicateRelationsCell.appendChild(duplicateRelationsButton);


    var deleteRow = infoTable.insertRow(-1);
    var deleteCell = deleteRow.insertCell(-1);

    var deleteButton = document.createElement("button");
    deleteButton.className = "btn btn-danger";

    deleteButton.innerHTML = "Delete";
    deleteButton.setAttribute("onclick", "cancelClass()");

    deleteCell.appendChild(deleteButton);

    deleteRow.insertCell(-1);

    d.appendChild(infoTable);
}

/**
 *
 * @param infoTable
 * @param index
 * @param desc
 * @param property
 */
function showInfoAddRow(infoTable, index, desc, property) {
    var row = infoTable.insertRow(index);

    var cell1 = row.insertCell(0);
    cell1.innerHTML = "<strong>" + desc + "</strong>";
    cell1.className = "capitalize";
    var cell2 = row.insertCell(1);
    cell2.innerHTML = property;
}