/**
 * options.js
 * pre-defined values for certain things like ontologies, filter options and more
 */

/**
 * the third party address to bypass CORS
 * @type {string}
 */
// uncomment the line if you need to use CORS
 //var cors = "https://cors-anywhere.herokuapp.com/";

/**
 * which URIs are considered an annotation property
 * @type {string[]}
 */
var annotation = [
    "http://www.w3.org/2000/01/rdf-schema#label",
    "http://www.w3.org/2000/01/rdf-schema#comment",
    "http://www.w3.org/2004/02/skos/core#prefLabel",
    "http://purl.org/dc/elements/1.1/title"
];

var orderByOptions = [
    "ASC",
    "DESC",
    "ASC(int)",
    "DESC(int)",
    "ASC(date)",
    "DESC(date)",
    "ASC(datetime)",
    "DESC(datetime)",
    "ASC(time)",
    "DESC(time)",
    "ASC(float)",
    "DESC(float)"
];

/**
 * aggregation functions
 * @type {string[]}
 */
var aggregationsOptions = [
    "COUNT",
    "SUM",
    "AVG",
    "MIN",
    "MAX"
];

/**
 * filter operators for attributes
 * @type {string[]}
 */
var filterOptions = [
    "contains",
    "is equal to",
    "not equal to",
    "starts with",
    "ends with",
    "regex",
    "=",
    "!=",
    "<",
    "<=",
    ">",
    ">="
];

/**
 * some pre-defined well known prefixes
 * @type {*[]}
 */
var prefixes = [
    {
        "prefix": "rdf",
        "uri": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "uriClass": "http://www.w3.org/1999/02/22-rdf-syntax-ns",
        "uriSPARQL": "<http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
    },
    {
        "prefix": "rdfs",
        "uri": "http://www.w3.org/2000/01/rdf-schema#",
        "uriClass": "http://www.w3.org/2000/01/rdf-schema",
        "uriSPARQL": "<http://www.w3.org/2000/01/rdf-schema#>"
    },
    {
        "prefix": "foaf",
        "uri": "http://xmlns.com/foaf/0.1/",
        "uriClass": "http://xmlns.com/foaf/0.1/",
        "uriSPARQL": "<http://xmlns.com/foaf/0.1/>"
    },
    {
        "prefix": "owl",
        "uri": "http://www.w3.org/2002/07/owl",
        "uriClass": "http://www.w3.org/2002/07/owl",
        "uriSPARQL": "<http://www.w3.org/2002/07/owl>"
    },
    {
        "prefix": "skos",
        "uri": "http://www.w3.org/2004/02/skos/core#",
        "uriClass": "http://www.w3.org/2004/02/skos/core",
        "uriSPARQL": "<http://www.w3.org/2004/02/skos/core#>"
    },
    {
        "prefix": "xsd",
        "uri": "http://www.w3.org/2001/XMLSchema#",
        "uriClass": "http://www.w3.org/2001/XMLSchema",
        "uriSPARQL": "<http://www.w3.org/2001/XMLSchema#>"
    }
];

/**
 * list of MRE ontologies
 * @type {*[]}
 */
var ontologies = [
    {
        "nameSpace": "http://mre.zcu.cz/ontology/mre.owl",
        "url": "https://mre.zcu.cz/ontology/mre.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/dasta.owl",
        "url": "https://mre.zcu.cz/ontology/dasta.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/dscl.owl",
        "url": "https://mre.zcu.cz/ontology/dscl.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/dcm.owl",
        "url": "https://mre.zcu.cz/ontology/dcm.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/form.owl",
        "url": "https://mre.zcu.cz/ontology/form.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/image.owl",
        "url": "https://mre.zcu.cz/ontology/image.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/image-mapping.owl",
        "url": "https://mre.zcu.cz/ontology/image-mapping.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/nihss.owl",
        "url": "https://mre.zcu.cz/ontology/nihss.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/pop.owl",
        "url": "https://mre.zcu.cz/ontology/pop.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/sits.owl",
        "url": "https://mre.zcu.cz/ontology/sits.owl",
        "xml": null
    },
    {
        "nameSpace": "http://mre.zcu.cz/ontology/stroke.owl",
        "url": "https://mre.zcu.cz/ontology/stroke.owl",
        "xml": null
    },
];

/**
 * a filter type with corresponding URI
 * @type {*[]}
 */
var datatypes = [
    {
        uri: "http://www.w3.org/2001/XMLSchema#string",
        type: "string"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#date",
        type: "date"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#dateTime",
        type: "date"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#boolean",
        type: "boolean"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#float",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#int",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#integer",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#decimal",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#long",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#short",
        type: "number"
    },
    {
        uri: "http://www.w3.org/2001/XMLSchema#double",
        type: "number"
    }
];