/**
 * reactor.js
 * contains functions for reactions of document elements
 */

function cancelClass() {

    var cls = classes[currentClass];

    var id = cls.id;

    removeNode(id);

    var isOrdered = isVariableOrdered(cls.name);

    if (isOrdered !== false) {

        orderBy.splice(isOrdered, 1);
    }

    cls.selected = false;
    cls.show = false;
    cls.aggrCount = false;

    // reset attributes
    for (var a = 0; a < cls.attributes.length; a++) {

        var attr = cls.attributes[a];

        // selected, clicked to false
        // show and optional to true
        // clear all filters and aggregations
        attr.selected = false;
        attr.filters = [];
        attr.clicked = false;
        attr.show = true;
        attr.optional = true;

        for (var f = 0; f < attr.aggregations.length; f++) {

            attr.aggregations[f].selected = false;

        }
    }

    // nothing is selected
    currentClass = -1;
    currentAttribute = -1;


    // update selections
    createClassSelection();
    createAllAttributesSelection();

    // clear the page
    document.getElementById("info").innerHTML = "";
    document.getElementById("attributes-selection").innerHTML = "";
    document.getElementById("object-properties-selection").innerHTML = "";
    document.getElementById("filter-button").innerHTML = "";
    document.getElementById("filter-table").innerHTML = "";
    document.getElementById("aggregations-selection").innerHTML = "";
    document.getElementById("attribute-table-div").innerHTML = "";
    document.getElementById("object-properties-table-div").innerHTML = "";


    showInfo();
    clearFiltersAndAggregations();
    restart();


    updateIds();

    generateQuery();

}

function cancelObjectProperty(i) {

    var id = parseInt(i);

    var objProperty = classes[currentClass].objectProperties[id];

    objProperty.selected = false;

    removeEdge(currentClass, classes[objProperty.target].id, objProperty.localName);

    restart();

    createObjectPropertiesSelection();
    createObjectPropertiesTable();

    generateQuery();

}


/**
 * function resets application state to the inicialization state
 */
function reset() {

    // clearing up duplicates
    for (var i = classes.length - 1; i >= 0; i--) {

        if (classes[i].duplicate === true) {
            classes.splice(i, 1);
            graph.nodes.splice(i, 1);
        } else {
            classes[i].duplicates = [];
        }

    }

    // resetting classes and attributes
    for (var j = 0; j < classes.length; j++) {

        // selected, SHOW and COUNT to default false
        var cls = classes[j];
        cls.selected = false;
        cls.show = false;
        cls.aggrCount = false;

        // reset attributes
        for (var a = 0; a < cls.attributes.length; a++) {

            var attr = cls.attributes[a];

            // selected, clicked to false
            // show and optional to true
            // clear all filters and aggregations
            attr.selected = false;
            attr.filters = [];
            attr.clicked = false;
            attr.show = true;
            attr.optional = true;

            for (var f = 0; f < attr.aggregations.length; f++) {

                attr.aggregations[f].selected = false;

            }


        }

        // set all object properties to not selected
        var objectProperties = [];

        for (var o = cls.objectProperties.length - 1; o >= 0; o--) {

            if (cls.objectProperties[o].target >= classes.length) {
            } else {
                cls.objectProperties[o].selected = false;
                objectProperties.push(cls.objectProperties[o]);
            }
        }

        cls.objectProperties = objectProperties;

    }

    clearFiltersAndAggregations();

    // nothing is selected
    currentClass = -1;
    currentAttribute = -1;

    // nothing is visualized
    dataset.nodes = [];
    dataset.edges = [];

    // nothing is sorted
    orderBy = [];

    // all additions to default
    query.LIMIT = 0;
    query.OFFSET = 0;
    query.DISTINCT = true;

    // update selections
    createClassSelection();
    createAllAttributesSelection();

    // clear the page
    document.getElementById("info").innerHTML = "";
    document.getElementById("attributes-selection").innerHTML = "";
    document.getElementById("object-properties-selection").innerHTML = "";
    document.getElementById("filter-button").innerHTML = "";
    document.getElementById("filter-table").innerHTML = "";
    document.getElementById("aggregations-selection").innerHTML = "";
    document.getElementById("attribute-table-div").innerHTML = "";
    document.getElementById("query-result").innerHTML = "";
    document.getElementById("results-table").innerHTML = "";
    document.getElementById("results-json").innerHTML = "";
    document.getElementById("object-properties-table-div").innerHTML = "";

    document.getElementById("offset-input").value = 0;
    document.getElementById("limit-input").value = 0;

    // update graph to reflect duplicate loss
    updateGraph();

    // clear generated query
    generateQuery();

    // clear visualization
    restart();

    printInfo("Application reset");


}

/**
 * toggles DISTINCT
 * @param checkbox
 */
function checkDistinct(checkbox) {
    query.DISTINCT = !query.DISTINCT;

    generateQuery();

}

/**
 * updates OFFSET
 * @param input
 */
function offsetChange(input) {
    query.OFFSET = parseInt(input.value);

    generateQuery();

}

/**
 * updates LIMIT
 * @param input
 */
function limitChange(input) {

    query.LIMIT = parseInt(input.value);

    generateQuery();

}

/**
 * cancels a selection of a aggregation by its index
 * @param index
 */
function cancelAggregation(index) {

    var id = parseInt(index);

    var attribute = classes[currentClass].attributes[currentAttribute];

    attribute.aggregations[id].selected = false;

    createAggregationsSelection(attribute);
    createAggregationsTable(attribute);

    var isOrdered = isVariableOrdered(attribute.aggregations[id].variable);

    if (isOrdered !== false) {

        orderBy.splice(isOrdered, 1);
    }

    generateQuery();

}

/**
 * reaction to ordered variable being pushed up
 * @param index
 */
function orderUp(index) {

    var id = parseInt(index);

    // it is already first - it cant go up
    if (id === 0) {
        return;
    }

    // if only one - no point
    if (orderBy.length === 1) {
        return;
    }

    swap(id, id - 1);

    createOrderByTable();

    generateQuery();
}

/**
 * reaction to ordered variable being pushed down
 * @param index
 */
function orderDown(index) {
    var id = parseInt(index);

    // cant go down
    if (id === orderBy.length - 1) {
        return;
    }

    if (orderBy.length === 1) {
        return;
    }

    swap(id, id + 1);

    createOrderByTable();

    generateQuery();

}

/**
 * removes ordered variable from being ordered
 * @param value
 */
function cancelOrder(value) {

    orderBy.splice(value, 1);

    generateQuery();


}

/**
 * selects a variable to be ordered (added to order table)
 * @param value
 */
function selectOrderOption(value) {

    var order = orderBy[value.id];

    order.selectedOrderOption = parseInt(value.value);

    generateQuery();

}

/**
 * removes a filter
 * @param value
 */
function cancelFilter(value) {

    classes[currentClass].attributes[currentAttribute].filters.splice(value, 1);

    createFilterTable(classes[currentClass].attributes[currentAttribute]);

    generateQuery();

}

/**
 * updates filter value with input from user
 * @param value
 */
function filterValue(value) {

    classes[currentClass].attributes[currentAttribute].filters[value.id].value = value.value;

    generateQuery();

}

/**
 * updates selected operator of a filter
 * @param select
 */
function selectOperator(select) {

    classes[currentClass].attributes[currentAttribute].filters[select.id].selectedOption = parseInt(select.value);

    generateQuery();
}

/**
 * adds a new filter (user clicked Add a filter button) based on attribute data type
 * @param attributeId
 */
function addFilter(attributeId) {

    var attribute = classes[currentClass].attributes[attributeId];

    var newFilter = {
        // id: attribute.filters.length,
        options: filterOptions,
        selectedOption: 0,
        value: ""
    };

    attribute.filters.push(newFilter);

    createFilterTable(attribute);


}

/**
 * removes attribute from selection
 * @param attributeId
 */
function cancelAttribute(attributeId) {

    clearFiltersAndAggregations();

    var attribute = classes[currentClass].attributes[attributeId];

    // if the attribute is ordered, need to handle that
    var isOrdered = isVariableOrdered(attribute.localName);

    if (isOrdered !== false) {
        orderBy.splice(isOrdered, 1);
    }

    // attribute filters, aggregations are reset
    // selected and clicked returns to false
    attribute.selected = false;
    attribute.clicked = false;
    attribute.filters = [];
    attribute.aggregations = [];

    currentAttribute = -1;

    createAttributesSelection();
    createAttributeTable();

    generateQuery();

}

/**
 * handles "selecting" attribute (clicking Select button)
 * clicking attribute means that filters and aggregations are prepared/shown
 * @param attributeId
 */
function addAttribute(attributeId) {

    // clicking on the same attribute currently selected
    if (currentAttribute === attributeId) {
        return;
    }

    clearFiltersAndAggregations();

    currentAttribute = attributeId;

    var attribute = classes[currentClass].attributes[currentAttribute];

    // un-click others
    for (var i = 0; i < classes[currentClass].attributes.length; i++) {
        classes[currentClass].attributes[i].clicked = false;
    }


    attribute.clicked = true;

    if (typeof attribute.filterType === "undefined") {
        assignFilterTypeTo(attribute);
    }


    createAttributeTable();
    createFilterSelection(attribute);
    createFilterTable(attribute);
    createAggregationsSelection(attribute);
    createAggregationsTable(attribute);

}

/**
 * toggles the COUNT of a class
 * @param checkBox
 */
function countCheckbox(checkBox) {

    var cls = classes[currentClass];

    cls.aggrCount = !cls.aggrCount;

    // beware it can be ordered
    var isOrdered = isVariableOrdered("count_" + cls.localName);

    if (isOrdered !== false) {
        orderBy.splice(isOrdered, 1);
    }

    generateQuery();
}

/**
 * toggles the SHOW of a class
 * @param checkBox
 */
function showCheckboxClass(checkBox) {

    var cls = classes[currentClass];

    cls.show = !cls.show;

    // beware it can be ordered
    var isOrdered = isVariableOrdered(cls.localName);

    if (isOrdered !== false) {
        orderBy.splice(isOrdered, 1);
    }

    generateQuery();
}

/**
 * toggles SHOW in attributes
 * @param checkBox
 */
function showCheckBox(checkBox) {

    var attribute = classes[currentClass].attributes[parseInt(checkBox.value)];

    attribute.show = !attribute.show;

    generateQuery();
}

/**
 * toggles OPTIONAL in attribute
 * @param checkBox
 */
function optionalCheckBox(checkBox) {

    var attribute = classes[currentClass].attributes[parseInt(checkBox.value)];

    attribute.optional = !attribute.optional;

    generateQuery();


}

/**
 * handles selection of an aggregation
 */
$('#aggregations-selection').on('select2:select', function (e) {

    var value = $(this).val();

    var attribute = classes[currentClass].attributes[currentAttribute];

    attribute.aggregations[value].selected = true;


    createAggregationsSelection(attribute);
    createAggregationsTable(attribute);

    generateQuery();

});

/**
 * handles selection of order selection
 */
$('#order-by-selection').on('select2:select', function (e) {

    var value = $(this).val();

    var order = {
        variable: value,
        orderOptions: orderByOptions,
        selectedOrderOption: 0
    };

    orderBy.push(order);

    createOrderByTable();

    generateQuery();


});

/**
 * reacts to an attribute being selected
 */
$('#attributes-selection').on('select2:select', function (e) {

    var value = $(this).val();

    var property = classes[currentClass].attributes[value];

    property.selected = true;

    createAttributeTable();
    createAttributesSelection();
    generateQuery();

});

/**
 * reacts to an object property is selected
 */
$('#object-properties-selection').on('select2:select', function (e) {

    var value = $(this).val();

    var cls = classes[currentClass];

    var selectedObjectProperty = cls.objectProperties[parseInt(value)];

    selectedObjectProperty.selected = true;

    var target = classes[selectedObjectProperty.target];

    var isSelected = isNodeSelected(target.id);

    // if the target node is already selected, add the new edge only
    if (isSelected) {

        var nwEdge = {
            source: cls.d3Id,
            target: target.d3Id,
            name: selectedObjectProperty.prefixedLocalName,
            edgeName: selectedObjectProperty.localName
        };

        addEdge(nwEdge);

        createObjectPropertiesSelection(currentClass);
        createObjectPropertiesTable();
        generateQuery();

        // else add both edge and node
    } else {
        var newNode = {
            id: target.id,
            name: target.prefix + ":" + target.localName
        };

        target.d3Id = dataset.nodes.length;
        target.selected = true;

        addNode(newNode);

        var newEdge = {
            source: cls.d3Id,
            target: target.d3Id,
            name: selectedObjectProperty.prefixedLocalName,
            edgeName: selectedObjectProperty.localName
        };

        addEdge(newEdge);

        createObjectPropertiesSelection(currentClass);
        createObjectPropertiesTable();
        generateQuery();
    }


});

/**
 * reacts to an attribute being selected
 * selection of all classes
 */
$('#select-attributes-selection').on('select2:select', function (e) {

    var value = $(this).val();

    var spl = value.split("/", 2);

    var cls = getClassByPrefixedLocalName(spl[0]);

    var attribute = cls.attributes[parseInt(spl[1])];

    attribute.selected = true;

    addNodeToViz(cls.id, true);


    createAllAttributesSelection();
    generateQuery();

});

/**
 * duplicates current class
 */
function duplicateClass(relations) {

    var cls = classes[currentClass];

    if (cls.duplicate === true) {
        cls = classes[cls.parent];
    }

    var duplicate = createDuplicateClass(cls);

    duplicate.d3Id = dataset.nodes.length;
    duplicate.id = classes.length;
    duplicate.duplicate = true;

    cls.duplicates.push(duplicate.id);

    duplicate.duplicateId = cls.duplicates.length;
    duplicate.localName += duplicate.duplicateId;
    duplicate.duplicates = [];
    duplicate.parent = cls.id;

    classes.push(duplicate);

    updateGraphWithDuplicate(cls.id, duplicate.id);

    var duplicateNode = {
        name: duplicate.prefixedLocalName + duplicate.duplicateId,
        id: duplicate.id
    };

    addNode(duplicateNode);

    if (relations === true) {
        addExistingEdgesToViz(cls.id, duplicateNode.id);
    }

    showInfo();


}


/**
 * adds a node to visualization
 * this function handles the required path to get to the new node by adding all required nodes and edges
 * @param value
 * @param attr if this was called from an attribute selection
 */
function addNodeToViz(value, attr) {

    var cls = classes[parseInt(value)];

    if (dataset.nodes.length === 0) {
        // first class ever

        var classNode = {
            name: cls.prefix + ":" + cls.localName,
            id: parseInt(value)
        };

        cls.selected = true;
        cls.d3Id = dataset.nodes.length;

        addNode(classNode);
    } else {

        if (cls.selected === true && attr === false) {
            // a new duplicate since class is already selected and it is not called from attribute selection

            var duplicate = createDuplicateClass(cls);

            duplicate.d3Id = dataset.nodes.length;
            duplicate.id = classes.length;
            duplicate.duplicate = true;

            cls.duplicates.push(duplicate.id);

            duplicate.duplicateId = cls.duplicates.length;
            duplicate.localName += duplicate.duplicateId;
            duplicate.duplicates = [];
            duplicate.parent = cls.id;

            classes.push(duplicate);

            updateGraphWithDuplicate(cls.id, duplicate.id);

            var duplicateNode = {
                name: duplicate.prefixedLocalName + duplicate.duplicateId,
                id: duplicate.id
            };

            addNode(duplicateNode);

            addExistingEdgesToViz(cls.id, duplicateNode.id);


            // the node is not selected yet
        } else {

            var checkD = checkDuplicates(cls);

            if (checkD !== false) {

                checkD.selected = true;

                var newN = {
                    name: checkD.prefix + ":" + checkD.localName,
                    id: checkD.id
                };

                checkD.d3Id = dataset.nodes.length;

                addNode(newN);

                addExistingEdgesToViz(cls.id, checkD.id);

                return;
            }

            // find possible path to get to new node
            var possiblePaths = findPathsToClass(value);

            // there is no path ?
            if (possiblePaths.length === 0) {

                // then we only add the new node if it is not selected
                if (cls.selected === false) {
                    var clas = {
                        name: cls.prefix + ":" + cls.localName,
                        id: parseInt(value)
                    };

                    cls.selected = true;
                    cls.d3Id = dataset.nodes.length;

                    addNode(clas);
                }

                // there is a path
            } else {

                // pick the first one - it is sorted by length
                var path = possiblePaths[0];
                // go along the path and add its nodes and edges if they are not selected
                for (var i = 0; i < path.length; i++) {

                    var edge = path[i];

                    // if source node is not selected, select
                    if (!isNodeSelected(edge.source)) {

                        var clsSource = {
                            name: classes[edge.source].prefix + ":" + classes[edge.source].localName,
                            id: edge.source
                        };

                        classes[edge.source].selected = true;
                        classes[edge.source].d3Id = dataset.nodes.length;

                        addNode(clsSource);
                    }

                    // if target node is not selected, select
                    if (!isNodeSelected(edge.target)) {

                        var clsTarget = {
                            name: classes[edge.target].prefix + ":" + classes[edge.target].localName,
                            id: edge.target
                        };

                        classes[edge.target].selected = true;
                        classes[edge.target].d3Id = dataset.nodes.length;

                        addNode(clsTarget);
                    }

                    var objProperty = getObjectPropertyBySourceAndTarget(edge.source, edge.target);

                    // if the relation is not selected, select it
                    if (objProperty.selected === false) {

                        var newEdge = {
                            source: classes[edge.source].d3Id,
                            target: classes[edge.target].d3Id,
                            name: objProperty.prefixedLocalName,
                            edgeName: objProperty.localName
                        };


                        objProperty.selected = true;
                        addEdge(newEdge);
                    }
                }
            }
        }

    }

}

function checkDuplicates(cls) {

    if (cls.duplicate === true) {
        return true;
    }

    var duplicates = cls.duplicates;

    for (var i = 0; i < duplicates.length; i++) {
        var duplicate = classes[duplicates[i]];

        if (duplicate.selected === false) {
            return duplicate;
        }

    }

    return false;

}

/**
 * reacting to selection of a class
 */
$('#select-classes-selection').on('select2:select', function (e) {

    var value = $(this).val();

    addNodeToViz(value, false);

    createClassSelection();
    generateQuery();


});

/**
 * function returns a list of paths to a target
 * there can be one path or multiple paths sorted by their length (the shortes first
 * @param target
 * @returns {*}
 */
function findPathsToClass(target) {

    var possiblePaths = [];

    for (var i = 0; i < dataset.nodes.length; i++) {

        var clsNode = dataset.nodes[i].id;

        var possiblePath = findPath(parseInt(clsNode), parseInt(target));
        if (possiblePath != null && possiblePath.length > 0) {
            possiblePaths.push(possiblePath);
        }

        var reverse = findPath(parseInt(target), parseInt(clsNode));
        if (reverse != null && reverse.length > 0) {
            possiblePaths.push(reverse);
        }
    }

    return possiblePaths.sort(compareByArrayLength);
}

/**
 * prints a message to html as a success
 * @param message
 */
function printSuccess(message) {
    document.getElementById("messenger").style.display = "";
    document.getElementById("messenger-danger").style.display = "none";
    document.getElementById("messenger-info").style.display = "none";
    document.getElementById("messenger-success").style.display = "";

    document.getElementById("messenger-success-body").innerHTML = message;
}

/**
 * prints a message to html as an error
 * @param message
 */
function printError(message) {
    document.getElementById("messenger").style.display = "";
    document.getElementById("messenger-danger").style.display = "";
    document.getElementById("messenger-info").style.display = "none";
    document.getElementById("messenger-success").style.display = "none";

    document.getElementById("messenger-danger-body").innerHTML = message;
}

/**
 * prints a message to html as a neutral information
 * @param message
 */
function printInfo(message) {
    document.getElementById("messenger").style.display = "";
    document.getElementById("messenger-danger").style.display = "none";
    document.getElementById("messenger-info").style.display = "";
    document.getElementById("messenger-success").style.display = "none";

    document.getElementById("messenger-info-body").innerHTML = message;
}